#include "qrs_attr.h"
#include "math.h"
#include <stdio.h>
#include <stdlib.h>

/* 
* przechowuje dane wejsciowe takie jak
* sygna i atrybuty dotyczce QRS, uzyskane
* we wczesniejszym etapie analizy sygnau
* a wykorzystywane obecnie. Przechowuje dugo
* tablic wczytanych danych oraz sygnau a take
* ilos wykrytych epizodw. 
*
* Tworzy globaln zmienna rejestr, ktra jest uzupeniana
* w trakcie prowadzonej w czasie dziaania programu analizy
* sygnau.
*/
struct STRegister{ 
       unsigned int dataLength; /* dlugosc tablicy z atrybutami QRS */
       unsigned int signalLength; /* dlugosc tablicy z sygnaem */
       QRS_ATTR *data; /*tablica z atrybutami QRS */
       unsigned char *signal; /* tablica z danymi sygnau */
       int epizodsAmount; /*ilosc epizodow */
} rejestr;

/* 
* przechowuje dane wynikowe przeprowadzonej analizy 
* 
*/
struct stAttr{ 
       char str[2*3];     /* rezultat obliczen ST
                              indeksy parzyste: poziom S-T w kanalach: 1, 2, 3 
                               [10uV]indeksy nieparzyste: nachylenie S-T w 
                               kanalach: 1, 2, 3  [10uV/s] */
       short chan[3];		/* Numer kanau, w ktrym epizod ST wystpi  */
       int  StartQrsNr;	/* Numer pierwszego zespolu QRS w obrebie epizodu */
       int  Dur;		/* Czas trwania epizodu [100 ms] */
       int  Sep;		/* Odstep od poprzedniego epizodu [100 ms] */
       char  DefNr;		/* numer def zawierajacej epizod  */
       unsigned char HRBefore;		/* wartosc HR przed rozpoczeciem epizodu ST */
       unsigned char HRAfter;		/* wartosc HR po zakonczeniu epizodu ST */
       unsigned char HRatMxD;		/* wartosc HR w chwili wystapienia maksimum 
                                   obnizenia/uniesienia  */            
};
/*
*                void wczytajAtrybutyQRS
*
* Przeznaczenie:
*               Wczytuje atrybuty QRS, ktre zawieraj wynik
*               analizy sygnau z wczeniejszej fazy projjektu
*
* Argumenty:
*           I char * filename - nazwa pliku do wczytania
            I/O STRegister rej - struktura rejestru
*
* Zwraca: void
* 
* Uywane funkcje: fopen, printf, rewind, fgets, fgetc,  fclose
*
* Uywane zmienne: FILE* file, int znak, long iloscDanych, int EOF, 
*                  QRS_ATTR data[], int iloscSpacji, char lineBuffer[80], 
*                  char pojedynczaDana[10], int iteratorPojDanej, char c,
*                  
*                  
*
* Uwagi: brak
*
* Autor: Jarosaw Kardynia i Micha Piela
*
* Ostatnia modyfikacja:   19 stycznia 2011r.
*/
void wczytajAtrybutyQRS(char * filename, STRegister rej){
     FILE* file;
	   
	 if ((file=fopen(filename, "r"))==NULL) {
		printf ("Nie mog otworzy pliku sygnau.\n");
		exit(1);
     }
     
	 int znak; // przechowuje pojedynczy znak wczytane z pliku
	 long iloscDanych=0; // ilosc znalezionych linii w pliku
	 
	 while((znak = fgetc( file ) ) != EOF ){ //sprawdza ilosc linii (moze zostac 
                                             //rozpoznane o 1 linie za malo
                                             //gdy plik nie konczy sie nowa pusta 
                                           //lini)
        if((char)znak == '\n'){	
			iloscDanych = iloscDanych +1;
		}
	 }
	 
	 rej.dataLength = iloscDanych; // zapisuje wynik w globalnym rejestrze

	 rewind(file);
	 
	 QRS_ATTR data[iloscDanych]; // alokuje pamiec dla odpowiedniej iloci danych
	 int iloscSpacji; // okresla numer slowa w zdaniu w danej linii (np. 0 
                      // oznacza ze analizowane jest pierwsze slowo)
     char lineBuffer[80]; // bufor na pojedyncza linie
     char pojedynczaDana[10]; // bufor na pojedyncza liczb
     int iteratorPojDanej; 
	 
     char c; // przechowuje pojedynczy znak
	 
     for(int iterator=0; iterator<10; iterator=iterator+1){//przygotowanie bufora 
                                                           // pojedycznej danej
         pojedynczaDana[iterator] = ' ';
         if(iterator == 9){
             pojedynczaDana[iterator] = '\0';
         }                                  
         
     }
	 
	 for(int i=0; i < rej.dataLength; i = i+1){ // pobiera kolejno now lini
		fgets(lineBuffer,80,file);
		
		//analizuje poj. linie
		for(int j=0, iloscSpacji=0, iteratorPojDanej=0; ;j = j+1){ 
			c = pojedynczaDana[iteratorPojDanej] = lineBuffer[j];
			
			if(c == '\0'){ // koniec linii
				     break;
			 }
            if(c == ' '){ // koniec wyrazu		
				if(iloscSpacji ==0){ //DetPt                               		    
					data[i].DetPt = atoi(pojedynczaDana);
				}else if(iloscSpacji ==1){ //AMax
					data[i].AMax = (char)atoi(pojedynczaDana);
				}else if(iloscSpacji ==2){ //localRR
					data[i].localRR = (short) atoi(pojedynczaDana);
				}else if(iloscSpacji ==3){ //HR
					data[i].HR = (unsigned char)atoi(pojedynczaDana);
				}else if(iloscSpacji ==4){ //MorfT
					data[i].MorfT = (char)atoi(pojedynczaDana);
				}else if(iloscSpacji ==5){ //cl
				      data[i].cl = (short)atoi(pojedynczaDana);
				}else if(iloscSpacji ==6){ //str[0]
					data[i].str[0] = (char)atoi(pojedynczaDana);
				}else if(iloscSpacji ==7){ //str[1]
					data[i].str[1] = (char)atoi(pojedynczaDana);
				}else if(iloscSpacji ==8){//str[2]
					data[i].str[2] = (char)atoi(pojedynczaDana);
				}else if(iloscSpacji ==9){//str[3]
					data[i].str[3] = (char)atoi(pojedynczaDana);
				}else if(iloscSpacji ==10){//str[4]
					data[i].str[4] = (char)atoi(pojedynczaDana);
				}else if(iloscSpacji ==11){//str[5]
					data[i].str[5] = (char)atoi(pojedynczaDana);
				}
				
				iloscSpacji = iloscSpacji+1;
				iteratorPojDanej= 0;
				
				//czyszczenie bufora pojedynczej danej
				for(int iterator=0; iterator<10; iterator=iterator+1){
                       pojedynczaDana[iterator] = ' ';
                       if(iterator == 9){
                           pojedynczaDana[iterator] = '\0';
                       }                                  
                }
				continue;
			}
			iteratorPojDanej = iteratorPojDanej+1;

		
		}
	 } 
	 rej.data = data; // przypisanie wczytanych danych do globalnego rejestru
	 fclose (file);
}
/*
*                void wczytajSygnal
*
* Przeznaczenie:
*               Wczytuje sygna
*
* Argumenty:
*           I char * filename - nazwa pliku do wczytania
            I/O STRegister rej - struktura rejestru
*
* Zwraca: void
* 
* Uywane funkcje: fopen, printf, fgetc, rewind, fgets, fclose
*
* Uywane zmienne: FILE* file, long iloscDanych, int EOF, unsigned char sig[],
*                  
* Uwagi: brak
*
* Autor: Jarosaw Kardynia i Micha Piela
*
* Ostatnia modyfikacja:   19 stycznia 2011r.
*/
void wczytajSygnal(char * filename, STRegister rej){
     FILE* file;
	   
	 if ((file=fopen(filename, "r"))==NULL) {
		printf ("Nie mog otworzy pliku sygnau.\n");
		exit(1);
     }
	 long iloscDanych=0;
	 while(fgetc( file )  != EOF ){ //sprawdza ilosc danych
		iloscDanych = iloscDanych +1;
	 }
	 rewind(file);
	 unsigned char sig[iloscDanych];

	 for(int i=0; i < iloscDanych; i = i+1){ // wczytuje dane
		sig[i] = (unsigned char)fgetc(file);
	 }	 
	 fclose (file);
	 
	 //zapis do struktury
	 rej.signal = sig;
	 rej.signalLength = iloscDanych;
	 printf("%i, ilosc", iloscDanych);
}
/* 
* int[] znajdzIzolinie
*
* szukanie lini izoelektrycznej
* poszukiwania zaczynaj si od punktu detekcji QRS wstecz w okresie czasu
* przypadajcego na maksymaln wartosc odcinka PQ = 100ms (PQ=40-100ms)
*/
int* znajdzIzolinie(STRegister rej,int i){
         int nrOstatniejProbkiPQ;
         int liniaIzo[3];
         int znaleziono=0;
         int punktDetekcjiQRS= rej.data[i].DetPt;
         int tolerancjaPredkosciLiniIz=2;
         int nrProbkiKoncowejPoszukiwania=punktDetekcjiQRS-0.1*120;
      
           while (1){
             
               for(int m= punktDetekcjiQRS;m>nrProbkiKoncowejPoszukiwania+1;m=m-1){
                   if ((rej.signal[m*3]-rej.signal[(m-1)*3])<tolerancjaPredkosciLiniIz &&(rej.signal[(m-1)*3]-rej.signal[(m-2)*3])<tolerancjaPredkosciLiniIz){
                      nrOstatniejProbkiPQ=m;
                         for(int chan=0;chan<3;chan=chan+1){
                                 liniaIzo[chan]=rej.signal[m*(chan+1)];
                                    
                         }
                      znaleziono=1;
                      break;
                   }                     
               }
             
               if (znaleziono==1) {break;}
               else {tolerancjaPredkosciLiniIz=tolerancjaPredkosciLiniIz+1;}
           }     
     return liniaIzo;
}

/*
*                void analizaST
*
* Przeznaczenie:
*               Dokonuje analizy sygnau EKG pod ktem znalezienia
*               nieprawidowosci w odcinku ST. Znajduje miejsca 
*               obnienia, podwyzszenia i nachylenia odcinka ST
*               korzystajc z danych uzyskanych z analizy sygnalu
*               z wczesniejszej fazy projektu.
*
* Argumenty:
*           I stAttr tab[] - przechowuje wynik analizy
            I/O STRegister rej - struktura rejestru
*
* Zwraca: void
* 
* Uywane funkcje: znajdzIzolinie, 
*
* Uywane zmienne:     int i,j,k; int RJtime;int RSTtime;int punktJ;int punktST;
*                      unsigned int DJtime;unsigned int DSTtime;int nrProbkaST;
*                      int nrProbkaJ;int STlength;int nrProbkiST_epizodu;
*                      unsigned int tolerancja;int epizodsAmount;stAttr s,
*                      int liniaIzo[], int amplitudaST; int amplitudaJ;
*                  
* Uwagi: brak
*
* Autor: Jarosaw Kardynia i Micha Piela
*
* Ostatnia modyfikacja:   19 stycznia 2011r.
*/
void analizaST(stAttr tab[], STRegister rej){ 
         
    int i,j,k;
    int RJtime;
    int RSTtime;
    int punktJ;
    int punktST;
    unsigned int DJtime;
    unsigned int DSTtime;
    int nrProbkaST;
    int nrProbkaJ;
    int STlength;
    int nrProbkiST_epizodu=0;
    unsigned int tolerancja=100;
    int epizodsAmount=0;
 
    stAttr s;
    //stAttr tabStr[rejestr.dataLength];
    for(i =0, k=0; i < 90; i = i+1) //rejestr.dataLength
    {
          
         int *liniaIzo = znajdzIzolinie(rejestr,i);
         int punktDetekcjiQRS= rej.data[i].DetPt;
         RJtime=40+0.948*sqrt(rej.data[i].localRR);
         RSTtime=40+2.21*sqrt(rej.data[i].localRR);
         DJtime=RJtime+rej.data[i].AMax; //w ms
         DSTtime=RSTtime+rej.data[i].AMax; // w ms
         nrProbkaST=punktDetekcjiQRS+DSTtime*128/1000;
          
         nrProbkaJ=punktDetekcjiQRS+DJtime*128/1000;
         STlength=RSTtime-RJtime;
         
         
          //TYLKO pochodzenia nadkomorowego
         if(rej.data[i].MorfT==0)
         {
                                  
             //amplitud punktw ST i J wzgledem lini izoel.
             int amplitudaST;  
             int amplitudaJ;
             for (int kanal=0;kanal<3;kanal=kanal+1){     
                             
                 amplitudaST=rej.signal[nrProbkaST*(kanal+1)]-liniaIzo[kanal];
                 amplitudaJ=rej.signal[nrProbkaJ*(kanal+1)]-liniaIzo[kanal];
                   
                 //maksymalne obnizenie, podniesienie
                  
                 if (abs(amplitudaST)>abs(amplitudaJ)) {s.str[kanal*2]=(char)amplitudaST;}
                 else {s.str[kanal*2]=(char)amplitudaJ;}
                  
                 //nachylenie
                 s.str[kanal*2+1]=(char)((amplitudaST-amplitudaJ)/(nrProbkaST-nrProbkaJ));  
                 //numer kanalu
                 s.chan[kanal]=kanal; 
                 
                 //WARTOSCI WSPOLNE DLA WSZYSTKICH KANALOW (OBLICZANE TYLKO RAZ)
                 if (kanal==2){  
                 //wartosci HR
                  
                     if (i==0) s.HRBefore=0;
                     else s.HRBefore=rej.data[i-1].HR;
                     s.HRAfter=rej.data[i+1].HR;
                     s.HRatMxD=rej.data[i].HR;
                
                     //czas trwania epizodu
                     s.Dur=STlength;
                       
                     //odstep od epizodu poprzedniego
                     if (  abs((int)s.str[0])>100 || abs((int)s.str[2])>100 || abs((int)s.str[4])>100) {
                        if(nrProbkiST_epizodu==0) { 
                           nrProbkiST_epizodu=nrProbkaST;
                           s.Sep=0;}
                        else{
                           s.Sep=(nrProbkaJ-nrProbkiST_epizodu)*1000/128;
                           nrProbkiST_epizodu=nrProbkaST;}  
                     } 
                     else{s.Sep=0;}
 
                     //Numer zespolu QRS przed epizodem
                     s.StartQrsNr=i;
                  }
             }   
        
        }//ednif   
          tab[k]=s;    
         k=k+1;  
    } 
     
    //rejestr.epizodsAmount=epizodsAmount;
}
void SteSave(stAttr tab[], char * filename)
{
     FILE* ste_attr;
     ste_attr=fopen(filename, "at");
     if(ste_attr == NULL){
          printf("Blad zapisu!");
          exit(1);            
     }
     int k;
     int nrEpizodu=1;
     for (k=0;k<90; k=k+1)
     {
         
         for (int kanal=0;kanal<3;kanal=kanal+1){
             if (abs((int)tab[k].str[kanal*2])>100)  {//jezeli jest epizod
                   fprintf(ste_attr, "%d ", tab[k].chan[kanal]);
                   fprintf(ste_attr, "%d ", tab[k].StartQrsNr);
                   fprintf(ste_attr, "%d ", tab[k].Dur);
                   fprintf(ste_attr, "%d ", tab[k].Sep);
                   fprintf(ste_attr, "%d ", tab[k].str[kanal*2]);
                   fprintf(ste_attr, "%d ", tab[k].str[kanal*2+1]);
                   fprintf(ste_attr, "%d ", nrEpizodu);
                   fprintf(ste_attr, "%d ", tab[k].HRBefore);
                   fprintf(ste_attr, "%d ", tab[k].HRAfter);
                   fprintf(ste_attr, "%d ", tab[k].HRatMxD);
                   fprintf(ste_attr, "\n");
                   
             }
            
         }
          nrEpizodu=nrEpizodu+1;
     }
     fclose(ste_attr);
     
}

int main(){
     wczytajAtrybutyQRS("qrs_attr.out", rejestr);
     wczytajSygnal("ah_21_3.dcm", rejestr);
     stAttr punktyST[rejestr.dataLength];
     
     analizaST(punktyST, rejestr);
     
     SteSave(punktyST, "ste_attr.out");
     return 0;
}


   
         
